% SOB-CS applied to Item-Distractor Similarity experiments 

function GenItemDistSim

global E   %experiment parameters
global C   %constants
global P   %model parameters
global M   %memory contents
global O   %output

warning('off', 'MATLAB:divideByZero');  %suppresses warnings about division by zero, which occurs frequently during computation of rehearsal accuracy

rand('state',124);  %initializes random generator
randn('state',124); 

%Complex span experimental parameters
E.trialnum = 50;
E.exps = [1,2,3,4];   %experiments to be simulated: 1 = reconstr. without distractors, 2 = reconstr including distractor, sim distractors following; 3 = like 2 with non-rhyming distr, 4 = reconstr, sim distractors preceding; 
E.recallmode = 1;     %1 = reconstruction from experimenter-defined candidate set (as in actual experiments), 2 = recall; candidate set = presented items and distr., 3 = recall; candidate set = all stimuli + presented distractors
E.setsize = [5, 4, 4, 4]; %for the 4 experiments
E.opduration = 1.0;
E.maxstor = 2*max(E.setsize);       %maximum list length 
E.opnum = 2;       %number of operations following each item
E.stimuli = [4,4]; %1 = digits, 2 = letters, 3 = non-rhyming words, 4 = rhyming words
E.enctime = 1.0;   %presentation time for an item 
E.simcond = 1;     %items: 1 = all dissimilar, 2 = all similar, 3 = alternating list of word items
C.itemdistsim = 1; %item-distractor similarity in the high-similarity condition = 1 (i.e., as high as similarity within a similar set, without discount for one being item, the other being distractor. Dissimilar condition is modelled by assigning similar distractors to non-presented-lure status
  
if E.recallmode == 3, P.c = 1.3; end  %performance is otherwise very bad

fid = fopen ('SOB.IDsim.out', 'w');

Serpos = zeros(E.trialnum, max(E.setsize)); %initialize serial-position variable 
DIntrus = zeros(1,E.trialnum); %initialize distractor-intrusion error variable 
NIntrus = zeros(1,E.trialnum); %initialize not-presented lure intrusion error variable 
O.SP = zeros(E.nreplic, 4, 2, max(E.setsize)); 
O.DIntrus = zeros(E.nreplic, 4, 2);  %distractor intrusions
O.NIntrus = zeros(E.nreplic, 4, 2);  %not-presented lure intrusions

%Generate vectors for position markers in context layer
M.cue = GenContext(C.cuesim, E.maxstor);

for exprm = E.exps
for idsim = 1:2
    if exprm == 3, E.Stimuli = [3,3]; end  %Exp. 3 uses non-rhyming distractor sets. 
    if exprm == 3, P.c = 0.3; if E.recallmode == 3, P.c = 1.0; end; end
    C.dsim(1,1) = C.sim(2); %within-burst similarity between distractors equals similarity between similar nonwords 
    C.dsim(2,1) = C.sim(1); %between-burst similarity.
    
    for id = 1:E.nreplic

        GenStimuli(E.stimuli(1), E.stimuli(2)); %creates stimuli (words) and distractors (words)

        for trial = 1:E.trialnum

            M.w = randn(C.cun, C.un).*P.nsubo;  %initialize weight matrix with random values
            
            %Generate list
            list = GenList(2*E.setsize(exprm));  %generates 2 lists of dissimilar words 
            simlist = list(1:E.setsize(exprm)); dissimlist = list(E.setsize(exprm)+1:end); %divide up into similar and dissimilar distractor/not-presented-lure list
            list = list(1:E.setsize(exprm));  % the list of memory items actually presented is the same as the list of similar distractors/not-presented lures generated above
            if idsim == 2,  %condition with similar distractors, dissimilar not-presented lures
                dlist = simlist;  %distractors
                npllist = dissimlist;  %not-presented lures
            else            %condition with dissimilar distractors, similar not-presented lures
                dlist = dissimlist; 
                npllist = simlist; 
            end
            if exprm ~= 4, distlist = dlist; nplurelist = npllist; end    %similar distractors following items 
            if exprm == 4, distlist = dlist([2:length(list), 1]); nplurelist = npllist([2:length(list), 1]); end  %similar distractors preceding items: distractors 1,2,3... are similar to list items 2,3,4, ... 1
            
            sel = randperm(E.opnum); %selection of distractor in each burst (1st or 2nd) to include in candidate set
            if E.recallmode == 1     %reconstruction, as in actual experiments
                if exprm == 1, 
                    M.candidates = M.stim(list,:);  %only list items are presented as recall options
                    M.recallset = list;
                else
                    M.candidates = [M.stim(list,:); squeeze(M.distr(distlist,sel(1),:)); squeeze(M.distr(nplurelist,sel(2),:))];
                    M.recallset = [list, -(30 + (1:E.setsize(exprm))), -(20 + (1:E.setsize(exprm)))];  %distractors marked by -31 to -34 (depending on position), non-presented lures by -21 to -24
                end
            end
            if E.recallmode == 2   %recall, candidate set = presented list items and distractors
               M.candidates = [M.stim(list,:); squeeze(M.distr(distlist, 1, :)); squeeze(M.distr(distlist, 2, :))];  
               M.recallset = [list, -(30 + (1:E.setsize(exprm))), -(30 + (1:E.setsize(exprm)))];
            end
            if E.recallmode == 3   %recall, candidate set = all items in stimulus set and distractors
                M.candidates = [M.stim; squeeze(M.distr(distlist, 1, :)); squeeze(M.distr(distlist, 2, :))];
                M.recallset = [1:size(M.stim,1), -(30 + (1:E.setsize(exprm))), -(30 + (1:E.setsize(exprm)))]; 
            end

            %Encoding and processing of distractors
            for item = 1:E.setsize(exprm)
                M.position = item;   %records current position in the list
                M.opnumber = 0; 
                GenEncoding(M.stim(list(item),:), E.enctime);
                for op = 1:E.opnum
                    M.opnumber = op;
                    GenProcess(distlist); 
                end
            end

            %Recall

            recall = zeros(1,5); 
            correct = zeros(1,5); 
            distinposition = zeros(1,E.setsize(exprm)); 
            distother = zeros(1,E.setsize(exprm)); 
            nplinposition = zeros(1,E.setsize(exprm)); 
            nplother = zeros(1,E.setsize(exprm)); 
            for probedpos = 1:E.setsize(exprm) 
                M.position = probedpos;
                [recall(probedpos), correct(probedpos)] = GenRecall(list, probedpos);   %calls the function for simulating recall
                distinposition(probedpos) = recall(probedpos) == -(30+probedpos);       %determines whether recalled stimulus is the distractor in the probed position
                distother(probedpos) = ismember(recall(probedpos), setdiff([-34:-31], -(30+probedpos)));   %determines whether it is another distractor
                nplinposition(probedpos) = recall(probedpos) == -(20+probedpos);                           %determines whether it is the NPL similar to the correct item (in low I-Dist-Sim condition)
                nplother(probedpos) = ismember(recall(probedpos), setdiff([-24:-21], -(20+probedpos)));    %determines whether it is another NPL
            end

            %collecting data
            Serpos(trial, 1:E.setsize(exprm)) = correct(1,1:E.setsize(exprm)); 
            DIntrus(trial) = (sum(distinposition)+sum(distother))/E.setsize(exprm);
            NIntrus(trial) = (sum(nplinposition)+sum(nplother))/E.setsize(exprm);

            %saving data
            if E.setsize(exprm) < 5
                list = [list, zeros(1,5-E.setsize(exprm))];  %extend list to maximum list length saved (= 5)
            end
                
            fprintf (fid, '%3.0f %1.2f %1.2f %1.0f %1.0f    %4.2f %4.2f %4.2f %4.2f %2.4f   %3.2f %3.2f %1.2f    %1.0f %1.0f %1.0f %1.0f %1.0f    %1.0f %1.0f %1.0f %1.0f %1.0f    %1.0f %1.0f %1.0f %1.0f %1.0f    %2.3f %2.3f %2.3f %2.3f  ', ...
                id, idsim, C.itemdistsim, E.opnum, exprm, ...
                P.nsubo, P.c, P.rate, P.tau, P.gain, ...
                mean(C.sim), C.cuesim, sel(1), ...
                list, recall, correct, mean(distinposition), mean(distother), mean(nplinposition), mean(nplother));
            fprintf (fid, ' \n'); 

        end  %trial

        O.SP(id, exprm, idsim, 1:E.setsize(exprm)) = mean(Serpos(:, 1:E.setsize(exprm)));
        O.NIntrus(id, exprm, idsim) = mean(NIntrus);
        O.DIntrus(id, exprm, idsim) = mean(DIntrus); 


    end  %id
end  %idsim
end  %exprm

O.MSP = squeeze(mean(O.SP,1)); 
O.MNI = squeeze(mean(O.NIntrus,1));
O.MDI = squeeze(mean(O.DIntrus,1));

if length(E.exps) < 3 || length(E.exps) == 4
    placement = [0.1, 0.55, 0.8, 0.35;
                0.1, 0.1, 0.8, 0.35];   %left, bottom, width, height
end
if length(E.exps) == 3
    placement = [0.1, 0.7, 0.8, 0.25;
                0.1, 0.38, 0.8, 0.25;   %left, bottom, width, height
                0.1, 0.06, 0.8, 0.25];
end
legtext = ['Low  ID-sim'; 'High ID-sim'];

% plot proportion correct by serial position

figure
PreFigure(0, placement(1,:));
plot(squeeze(O.MSP(E.exps(1),:,1:E.setsize(1)))');
legend(legtext);
title('SPC, Exp. 1: D. after sim. items, ROO');
PostFigure([0, E.setsize(1)+1, 0, 1], ' ', 'P(recall)');

PreFigure(0, placement(2,:));
plot(squeeze(O.MSP(E.exps(2),:,1:E.setsize(2)))');
legend(legtext);
title('Exp. 2: D. after sim. items, ROIO');
PostFigure([0, E.setsize(2)+1, 0, 1], 'Serial Position ', 'P(recall)');    

if length(E.exps) > 3, 
    figure
    placement(3:4,:) = placement(1:2,:); 
end

PreFigure(0, placement(3,:));
plot(squeeze(O.MSP(E.exps(3),:,1:E.setsize(3)))');
legend(legtext);
title('Exp. 3: D. before sim. items, ROIO');
PostFigure([0, E.setsize(3)+1, 0, 1], ' ', 'P(recall)');   

PreFigure(0, placement(4,:));
plot(squeeze(O.MSP(E.exps(4),:,1:E.setsize(4)))');
legend(legtext);
title('Exp. 4: D. after sim. items, non-rhyming, ROIO');
PostFigure([0, E.setsize(4)+1, 0, 1], 'Serial Position ', 'P(recall)');    


fclose(fid);




