% SOB applied to Item-Distractor Similarity experiments 1-3 in Bristol
% Long-List version: each distractor receives its own position marker

function GenItemDistSimLL(model)

global E   %experiment parameters
global C   %constants
global P   %model parameters
global M   %memory contents
global O   %output

clear PC; clear MeanPC;
warning('off', 'MATLAB:divideByZero');  %suppresses warnings about division by zero, which occurs frequently during computation of rehearsal accuracy

rand('state',124);  %initializes random generator
randn('state',124); 
%rand('state',sum(100*clock));  %initializes random generator
%randn('state',(100*clock)); 

%Complex span experimental parameters
E.trialnum = 50;
E.exps = [1,2,3,4];   %experiments to be simulated: 1 = ROO without distractors, 2 = ROIO (including distractors), sim distractors following; 3 = ROIO, sim distractors preceding; 4 = like 2 with non-rhyming distr. 
E.maxstor = 15;       %maximum list length (= storage demand)
E.setsize = [5, 4, 4, 4]; %for the 4 experiments
E.opduration = 1.0;
E.freetime = 0; 
E.opnum = 2;             %number of operations following each item
E.stimuli = [4,4]; %1 = digits, 2 = letters, 3 = non-rhyming words, 4 = rhyming words
E.enctime = 1.0;   %presentation time for an item 
E.simcond = 1;     %items: 1 = all dissimilar, 2 = all similar, 3 = alternating list of word items
E.distvar = 0;     %irrelevant 
C.itemdistsim = 1;   %item-distractor similarity in the high-similarity condition = 1 (i.e., as high as similarity within a similar set, without discount for one being item, the other being distractor. Dissimilar condition is modelled by assigning similar distractors to non-presented-lure status

C.sim = [0.5, 0.6];  %reduced similarity contrast to get closer to the size of the empirical effect
%P.c = 0.3;  %reduced discriminability of stimuli because they are non-words
P.c = 0.45;

fid = fopen ('SOB.IDsim.out', 'w');

Serpos = zeros(E.trialnum, max(E.setsize)); %initialize serial-position variable 
DIntrus = zeros(1,E.trialnum); %initialize distractor-intrusion error variable 
NIntrus = zeros(1,E.trialnum); %initialize not-presented lure intrusion error variable 
O.SP = zeros(E.nreplic, 4, 2, max(E.setsize)); 
O.DIntrus = zeros(E.nreplic, 4, 2);  %distractor intrusions
O.NIntrus = zeros(E.nreplic, 4, 2);  %not-presented lure intrusions


%Generate vectors for position markers in context layer
M.cue = GenContext(C.cuesim, E.maxstor);

for exprm = E.exps
for idsim = 1:2
    if exprm == 4, E.Stimuli = [3,3]; end  %Exp. 4 uses non-rhyming distractor sets. 
    %if exprm == 4, C.sim = [0.50, 0.60]; end %reduce similarity when nonrhyming distractors are used
    if exprm == 4, P.c = 0.3; end
    if model == 4, P.c = P.c * 1.5; end %when energy-gating is switched off, performance drops overall - compensate by higher P.c
    C.dsim(1,1) = C.sim(2); %within-burst similarity between distractors equals similarity between similar nonwords 
    C.dsim(2,1) = C.sim(1); %between-burst similarity.
    %if exprm == 4, C.dsim(1,1) = 0.55; end  %within-burst similarity between distractors is reduced because the 2 distractors only share a vowel
    
    for id = 1:E.nreplic

        M.w = randn(C.cun, C.un).*P.nsubo;  %for SOB encoding to cues, assign randomised values in weights matrix

        for trial = 1:E.trialnum
            
            GenStimuli(E.stimuli(1), E.stimuli(2)); %creates stimuli (words) and distractors (words)
            M.w = P.PI*M.w; 
            if P.PI == 0, M.w = randn(C.cun, C.un).*P.nsubo; end %for the standard no-PI version of SOB

            %Generate list
            list = GenList(2*E.setsize(exprm), E.stimuli(1));  %generates 2 lists of dissimilar words 
            simlist = list(1:E.setsize(exprm)); dissimlist = list(E.setsize(exprm)+1:end); %divide up into similar and dissimilar distractor/not-presented-lure list
            list = list(1:E.setsize(exprm));  
            if idsim == 2,  %condition with similar distractors, dissimilar not-presented lures
                dlist = simlist;  %distractors
                npllist = dissimlist;  %not-presented lures
            else
                dlist = dissimlist; 
                npllist = simlist; 
            end
            if exprm ~= 3, distlist = dlist; nplurelist = npllist; end    %similar distractors following items 
            if exprm == 3, distlist = dlist([2:length(list), 1]); nplurelist = npllist([2:length(list), 1]); end  %similar distractors preceding items: distractors 1,2,3... are similar to list items 2,3,4, ... 1
            sel = randperm(E.opnum); %selection of distractor in each burst (1st or 2nd) to include in candidate set
            if exprm == 1, 
                M.candidates = M.stim(list,:);  %only list items are presented as recall options
                M.recallset = list;
            else
                M.candidates = [M.stim(list,:); squeeze(M.distr(distlist,sel(1),:)); squeeze(M.distr(nplurelist,sel(2),:))];
                M.recallset = [list, -(30 + (1:E.setsize(exprm))), -(20 + (1:E.setsize(exprm)))];  %distractors marked by -31 to -34 (depending on position), non-presented lures by -21 to -24
            end

            %Reorganization of M.distr so that GenProcess can be applied and pick out the right distractor, despite the fact that
            %distractors get their own position markers, so M.position no longer serves to identify the distractors in the original M.distr
            Distr = zeros(3*E.setsize(exprm), 2, C.un); 
            n = 0;
            for i = 1:E.setsize(exprm)
                n = n+1;
                for o = 1:2
                    n = n+1;
                    Distr(n, o, :) = squeeze(M.distr(distlist(i), o, :));
                end
            end
            M.distr = Distr;
            distlist = 1:3*E.setsize(exprm);
            
            %Encoding and processing of distractors
            M.position = 0; 
            for item = 1:E.setsize(exprm)
                M.position = M.position + 1;   %records current position in the list
                M.opnumber = 0; 
                GenEncoding(M.stim(list(item),:), E.enctime);
                for op = 1:E.opnum
                    M.opnumber = op;
                    M.position = M.position + 1;
                    GenProcess(list, 1, 1, distlist); 
                end
            end

            %Recall

            recall = zeros(1,5); 
            correct = zeros(1,5); 
            distinposition = zeros(1,E.setsize(exprm)); 
            distother = zeros(1,E.setsize(exprm)); 
            nplinposition = zeros(1,E.setsize(exprm)); 
            nplother = zeros(1,E.setsize(exprm)); 
            M.position = 1; 
            for probedpos = 1:E.setsize(exprm) 
                [recall(probedpos), correct(probedpos)] = GenRecall(list, probedpos);
                M.position = M.position + 3;  %skip over the two distractors
                %r = find(M.recallset == recall(probedpos)); %identifies the recalled item's position in recallset
                %M.candidates(r,:) = []; %removes the row r from candidate set (for experiment in which recalled items are eliminated from screen)
                %M.recallset(r) = [];    %removes the entry r from reallset
                distinposition(probedpos) = recall(probedpos) == -(30+probedpos);
                distother(probedpos) = ismember(recall(probedpos), setdiff([-34:-31], -(30+probedpos)));
                nplinposition(probedpos) = recall(probedpos) == -(20+probedpos);
                nplother(probedpos) = ismember(recall(probedpos), setdiff([-24:-21], -(20+probedpos)));
            end

            %collecting data
            Serpos(trial, 1:E.setsize(exprm)) = correct(1,1:E.setsize(exprm)); 
            DIntrus(trial) = (sum(distinposition)+sum(distother))/E.setsize(exprm);
            NIntrus(trial) = (sum(nplinposition)+sum(nplother))/E.setsize(exprm);

            %saving data
            if E.setsize(exprm) < 5
                list = [list, zeros(1,5-E.setsize(exprm))];  %extend list to maximum list length saved (= 5)
            end
                
            fprintf (fid, '%3.0f %2.0f %1.2f %1.2f %1.2f %1.0f %1.0f %1.0f     %1.0f %1.0f %4.2f %4.2f %4.2f %4.2f %4.2f %4.2f %2.4f   %3.2f %3.2f %1.2f    %1.0f %1.0f %1.0f %1.0f %1.0f    %1.0f %1.0f %1.0f %1.0f %1.0f    %1.0f %1.0f %1.0f %1.0f %1.0f    %2.3f %2.3f %2.3f %2.3f  ', ...
                id, model, idsim, C.itemdistsim, E.freetime, E.opnum, exprm, E.simcond, ...
                P.contextstyle, P.recallset, P.threshold, P.nsubo, P.c, P.rate, P.removalrate, P.tau, P.gain, ...
                mean(C.sim), C.cuesim, C.rectime, ...
                list, recall, correct, mean(distinposition), mean(distother), mean(nplinposition), mean(nplother));
            fprintf (fid, ' \n'); 

        end  %trial

        O.SP(id, exprm, idsim, 1:E.setsize(exprm)) = mean(Serpos(:, 1:E.setsize(exprm)));
        O.NIntrus(id, exprm, idsim) = mean(NIntrus);
        O.DIntrus(id, exprm, idsim) = mean(DIntrus); 


    end  %id
end  %idsim
end  %exprm

O.MSP = squeeze(mean(O.SP,1)); 
O.MNI = squeeze(mean(O.NIntrus,1));
O.MDI = squeeze(mean(O.DIntrus,1));

if length(E.exps) < 3 || length(E.exps) == 4
    placement = [0.1, 0.55, 0.8, 0.35;
                0.1, 0.1, 0.8, 0.35];   %left, bottom, width, height
end
if length(E.exps) == 3
    placement = [0.1, 0.7, 0.8, 0.25;
                0.1, 0.38, 0.8, 0.25;   %left, bottom, width, height
                0.1, 0.06, 0.8, 0.25];
end
legtext = ['Low  ID-sim'; 'High ID-sim'];

if E.stimuli(2) == 3  %non-rhyming items and distractors
    IDsim = [sqrt(C.sim(2))*sqrt((C.sim(1)/C.sim(2)))*C.itemdistsim*sqrt(C.dsim(1)), sqrt(C.sim(2))*C.itemdistsim*sqrt(C.dsim(1))];  %actual ID-similarity values for dissimilar and similar distractors
    %ID-sim for dissimilar d's = sim(item1,proto1)*sim(proto1,proto2)*sim(proto2,distproto2)*sim(distproto2,dist2)
end
if E.stimuli(2) == 4 %rhyming items and distractors
   IDsim = [C.sim(2)*sqrt((C.sim(1)/C.sim(2)))*C.itemdistsim*C.dsim(1), min(C.sim(2), C.dsim(1))*C.itemdistsim];  
   %ID-sim for dissimilar computed as for non-rhyming words except that
   %sim(item1,proto1) is not sqrt(C.sim(2)) but just C.sim(2), and sim(distproto2,dist2) is E.dsim(1). 
   %ID-sim for similar d's = min(proportion of vector-initial features
   %kept)*itemdistsim, where itemdistsim = probability of keeping one of
   %the features of the min(x) section of the vector
end

% plot proportion correct by serial position

figure
PreFigure(0, placement(1,:));
plot(squeeze(O.MSP(E.exps(1),:,1:E.setsize(1)))');
legend(legtext);
title('SPC, Exp. 1: D. after sim. items, ROO');
PostFigure([0, E.setsize(exprm)+1, 0, 1], ' ', 'P(recall)');

PreFigure(0, placement(2,:));
plot(squeeze(O.MSP(E.exps(2),:,1:E.setsize(2)))');
legend(legtext);
title('Exp. 2: D. after sim. items, ROIO');
PostFigure([0, E.setsize(exprm)+1, 0, 1], ' ', 'P(recall)');    

if length(E.exps) > 3, 
    figure
    placement(3:4,:) = placement(1:2,:); 
end

PreFigure(0, placement(3,:));
plot(squeeze(O.MSP(E.exps(3),:,1:E.setsize(3)))');
legend(legtext);
title('Exp. 3: D. before sim. items, ROIO');
PostFigure([0, E.setsize(exprm)+1, 0, 1], 'Serial Position', 'P(recall)');   

PreFigure(0, placement(4,:));
plot(squeeze(O.MSP(E.exps(4),:,1:E.setsize(4)))');
legend(legtext);
title('Exp. 4: D. after sim. items, non-rhyming, ROIO');
PostFigure([0, E.setsize(exprm)+1, 0, 1], ' ', 'P(recall)');    

% plot NPL-Intrusions and Distractor-Intrusions by Item-Distractor Similarity
figure
PreFigure(0, placement(1,:));
plot(IDsim', [O.MNI(1,:); O.MDI(1,:)]');
legend('Extralist', 'Distractor');
title('Intrusions, Exp. 1: D. after sim. items, ROO');
PostFigure([0, 1, 0, 0.5], ' ', 'P(Intrusion)');

PreFigure(0, placement(2,:));
plot(IDsim', [O.MNI(2,:); O.MDI(2,:)]');
title('Exp. 2: D. after sim. items, ROIO');
PostFigure([0, 1, 0, 0.5], ' ', 'P(Intrusion)');    

if length(E.exps) > 3, 
    figure
    placement(3:4,:) = placement(1:2,:); 
end

PreFigure(0, placement(3,:));
plot(IDsim', [O.MNI(3,:); O.MDI(3,:)]');
title('Exp. 3: D. before sim. items, ROIO');
PostFigure([0, 1, 0, 0.5], 'Item-Distractor Similarity', 'P(Intrusion)');   

PreFigure(0, placement(4,:));
plot(IDsim', [O.MNI(4,:); O.MDI(4,:)]');
title('Exp. 4: D. after sim. items, non-rhyming, ROIO');
PostFigure([0, 1, 0, 0.5], 'Item-Distractor Similarity', 'P(Intrusion)');   

fclose(fid);




